@@ -1,4 +1,3 @@
-import datetime
 import json
 import os
 import traceback
@@ -188,12 +187,14 @@ def serialise_event_stages(save_data, event_stages):
     return save_data
 
 
-def serialise_wierd_plat_ticket_data(save_data, data):
-    save_data = write(save_data, data["total"])
-    for dict in data["Value"]:
-        save_data = write(save_data, dict["unknown_48"])
-        save_data = serialise_utf8_string(save_data, dict["string"])
-        save_data = write(save_data, dict["unknown_49"])
+def serialse_purchase_receipts(save_data, data):
+    save_data = write(save_data, len(data), 4)
+    for dict in data:
+        save_data = write(save_data, dict["unknown_4"], 4)
+        save_data = write(save_data, len(dict["item_packs"]), 4)
+        for string_dict in dict["item_packs"]:
+            save_data = serialise_utf8_string(save_data, string_dict["Value"])
+            save_data = write(save_data, string_dict["unknown_1"], 1)
     return save_data
 
 
@@ -204,9 +205,7 @@ def serialise_dumped_data(save_data, data):
 
 
 def serialise_outbreaks(save_data, outbreaks):
-    data = outbreaks["data"]
     outbreak_data = outbreaks["outbreaks"]
-    save_data = serialise_dumped_data(save_data, data)
 
     save_data = write(save_data, len(outbreak_data), 4)
     for chapter_id in outbreak_data:
@@ -381,7 +380,7 @@ def export_json(save_stats, path):
     ordered_data = parse_save.re_order(save_stats)
     if os.path.isdir(path):
         path = os.path.join(path, "save_data.json")
-    f = open(path, "w")
+    f = open(path, "w", encoding="utf-8")
     f.write(json.dumps(ordered_data, indent=4))
     helper.coloured_text(f"Successfully wrote json to &{os.path.abspath(path)}&")
 
@@ -408,15 +407,36 @@ def serialise_medals(save_data, medals):
 
 
 def serialise_play_time(save_data, play_time):
-    time_data = play_time.split(":")
-    hours = int(time_data[0])
-    minutes = int(time_data[1])
-    seconds = int(time_data[2])
-    delta = datetime.timedelta(hours=hours, minutes=minutes, seconds=seconds)
-    value = delta.total_seconds() * 30
-    save_data = write(save_data, value, 4)
+    frames = helper.hmsf_seconds(play_time)
+    save_data = write(save_data, frames, 4)
     return save_data
 
+def serialise_mission_segment(save_data, data):
+    save_data = write(save_data, len(data), 4)
+    for mission in data:
+        save_data = write(save_data, mission, 4)
+        save_data = write(save_data, data[mission], 4)
+    return save_data
+
+def serialise_missions(save_data, missions):
+    save_data = serialise_mission_segment(save_data, missions["flags"])
+    save_data = serialise_mission_segment(save_data, missions["values"])
+    return save_data
+
+def serialise_dojo(save_data, dojo_data):
+    save_data = write(save_data, len(dojo_data), 4)
+    for subchapter_id in dojo_data:
+        subchapter_data = dojo_data[subchapter_id]
+
+        save_data = write(save_data, subchapter_id, 4)
+        save_data = write(save_data, len(subchapter_data), 4)
+
+        for stage_id in subchapter_data:
+            score = subchapter_data[stage_id]
+            
+            save_data = write(save_data, stage_id, 4)
+            save_data = write(save_data, score, 4)
+    return save_data
 
 def start_serialize(save_stats):
     try:
@@ -517,10 +537,10 @@ def serialize_save(save_stats):
     save_data = write_length_data(
         save_data, save_stats["unknown_12"], write_length=False)
 
-    save_data = write_length_data(
-        save_data, save_stats["cat_storage_id"], 2, 4)
-    save_data = write_length_data(
-        save_data, save_stats["cat_storage_type"], 2, 4, False)
+    if save_stats["cat_storage"]["len"]:
+        save_data = write(save_data, len(save_stats["cat_storage"]["ids"]), 2)
+    save_data = write_length_data(save_data, save_stats["cat_storage"]["ids"], 2, 4, False)
+    save_data = write_length_data(save_data, save_stats["cat_storage"]["types"], 2, 4, False)
 
     save_data = serialise_event_stages_current(
         save_data, save_stats["event_current"])
@@ -528,8 +548,8 @@ def serialize_save(save_stats):
 
     save_data = write_length_data(
         save_data, save_stats["unknown_15"], write_length=False)
-    save_data = write_length_data(save_data, save_stats["unknown_16"])
 
+    save_data = write_length_data(save_data, save_stats["unit_drops"])
     save_data = write(save_data, save_stats["rare_gacha_seed"])
 
     save_data = write(save_data, save_stats["unknown_17"])
@@ -538,19 +558,24 @@ def serialize_save(save_stats):
     save_data = serialise_time_data(
         save_data, save_stats["second_time"], save_stats["dst"], save_stats["dst_val"])
 
-    save_data = write(save_data, save_stats["unknown_105"])
-    save_data = write_length_data(save_data, save_stats["unknown_106"], 4, 4, False, 4)
-    save_data = write(save_data, save_stats["unknown_107"])
-
-    save_data = write(save_data, save_stats["unknown_108"]["flag"])
-    save_data = write(save_data, save_stats["unknown_111"])
-    save_data = serialise_utf8_string(save_data, save_stats["unknown_110"])
-    save_data = write(save_data, save_stats["unknown_108"]["length"])
-    for i in range(len(save_stats["unknown_108"]["Value"])):
-        save_data = serialise_utf8_string(save_data, save_stats["unknown_108"]["Value"][i])
+    save_data = write_length_data(save_data, save_stats["unknown_105"], 4, 4, False)
     
-    save_data = write(save_data, save_stats["unknown_109"])
+    save_data = write(save_data, save_stats["unknown_107"])
+    if save_stats["game_version"]["Value"] < 110500 or save_stats["dst"]:
+        save_data = serialise_utf8_string(save_data, save_stats["unknown_110"])
+    save_data = write(save_data, len(save_stats["unknown_108"]), 4)
+    for i in range(len(save_stats["unknown_108"])):
+        save_data = serialise_utf8_string(save_data, save_stats["unknown_108"][i])
+    
+    if save_stats["dst"]:
+        unknown_112 = save_stats["unknown_112"]
+        for float in unknown_112["floats"]:
+            save_data = write(save_data, float)
+        save_data = write(save_data, len(unknown_112["strs"]), 4)
+        save_data = serialise_dumped_data(save_data, unknown_112["data"])
 
+    if not save_stats["dst"]:
+        save_data = write(save_data, save_stats["unknown_111"])
     save_data = write(save_data, save_stats["unlocked_slots"])
 
     save_data = write(save_data, save_stats["unknown_20"]["Length_1"], 4)
@@ -560,6 +585,13 @@ def serialize_save(save_stats):
     save_data = write_length_data(
         save_data, save_stats["unknown_21"], write_length=False)
 
+
+    save_data = write(save_data, save_stats["trade_progress"])
+
+    save_data = write_length_data(
+        save_data, save_stats["unknown_24"], write_length=False)
+
+
     save_data = write_length_data(
         save_data, save_stats["catseye_related_data"])
 
@@ -584,11 +616,11 @@ def serialize_save(save_stats):
     save_data = write_length_data(
         save_data, save_stats["stage_data_related_1"], 4, 1, False, length)
 
-    lengths = save_stats["stage_data_related_2"]["Lengths"]
+    lengths = save_stats["event_timed_scores"]["Lengths"]
     length = lengths[0] * lengths[1] * lengths[2]
     save_data = write_length_data(save_data, lengths, write_length=False)
     save_data = write_length_data(
-        save_data, save_stats["stage_data_related_2"], 4, 4, False, length)
+        save_data, save_stats["event_timed_scores"], 4, 4, False, length)
 
     save_data = serialise_utf8_string(save_data, save_stats["inquiry_code"])
     save_data = serialise_play_time(save_data, save_stats["play_time"])
@@ -651,8 +683,8 @@ def serialize_save(save_stats):
     save_data = write(save_data, save_stats["unknown_47"])
 
     save_data = write(save_data, save_stats["gv_54"])
-    save_data = serialise_wierd_plat_ticket_data(
-        save_data, save_stats["wierd_plat_ticket_data"])
+    save_data = serialse_purchase_receipts(
+        save_data, save_stats["purchases"])
     save_data = write(save_data, save_stats["gv_54"])
     save_data = write(save_data, save_stats["gamatoto_skin"])
     save_data = write(save_data, save_stats["platinum_tickets"])
@@ -665,6 +697,15 @@ def serialize_save(save_stats):
     save_data = write(save_data, save_stats["gv_55"])
 
     save_data = write(save_data, save_stats["unknown_51"])
+    save_data = serialise_dumped_data(save_data, save_stats["unknown_113"])
+
+    save_data = serialise_dojo(save_data, save_stats["dojo_data"])
+
+    save_data = write(save_data, save_stats["unknown_114"])
+
+    save_data = write(save_data, save_stats["gv_58"])
+
+    save_data = write(save_data, save_stats["unknown_115"])
 
     save_data = serialise_outbreaks(save_data, save_stats["outbreaks"])
 
@@ -685,6 +726,7 @@ def serialize_save(save_stats):
 
     save_data = serialise_dumped_data(save_data, save_stats["unknown_59"])
     save_data = serialise_tower(save_data, save_stats["tower"])
+    save_data = serialise_missions(save_data, save_stats["missions"])
     save_data = serialise_dumped_data(save_data, save_stats["unknown_60"])
     save_data = serialise_dumped_data(save_data, save_stats["unknown_61"])
     save_data = write(save_data, save_stats["challenge"]["Score"])
@@ -746,8 +788,11 @@ def serialize_save(save_stats):
     save_data = serialise_dumped_data(save_data, save_stats["unknown_76"])
 
     save_data = write(save_data, save_stats["gv_80700"])
+    if save_stats["dst"]:
+        if save_stats["gv_100600"]["Value"] == 100600:
+            save_data = write(save_data, save_stats["unknown_104"])
+            save_data = write(save_data, save_stats["gv_100600"])
 
-    save_data = write(save_data, save_stats["unknown_104"])
 
     save_data = write(save_data, save_stats["restart_pack"])
     save_data = serialise_dumped_data(save_data, save_stats["unknown_101"])
